@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2011 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2017 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -1681,6 +1681,9 @@ static const struct {
 	{C_GEORGIAN_PS,		CS_GEORGIAN_PS},
 	{C_TCVN5712_1,		CS_TCVN5712_1},
 	{C_ISO_8859_16,		CS_ISO_8859_16},
+	{C_UTF_16,		CS_UTF_16},
+	{C_UTF_16BE,		CS_UTF_16BE},
+	{C_UTF_16LE,		CS_UTF_16LE},
 };
 
 static const struct {
@@ -2357,7 +2360,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 
 				mb_len = g_utf8_skip[*(guchar *)p];
 
-				Xstrndup_a(part_str, srcp, cur_len + mb_len, );
+				part_str = g_strndup(srcp, cur_len + mb_len);
 				out_str = conv_codeset_strdup_full
 					(part_str, src_encoding, block_encoding,
 					 &error);
@@ -2376,6 +2379,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 							((guchar *)out_str);
 
 				g_free(out_str);
+				g_free(part_str);
 
 				if (mimestr_len + strlen(block_encoding) + out_enc_str_len <= left) {
 					cur_len += mb_len;
@@ -2394,8 +2398,8 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 			}
 
 			if (cur_len > 0) {
-				Xstrndup_a(part_str, srcp, cur_len, );
 				error = 0;
+				part_str = g_strndup(srcp, cur_len);
 				out_str = conv_codeset_strdup_full
 					(part_str, src_encoding, block_encoding,
 					 &error);
@@ -2413,7 +2417,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 						qp_get_q_encoding_len
 							((guchar *)out_str);
 
-				Xalloca(enc_str, out_enc_str_len + 1, );
+				enc_str = g_malloc(out_enc_str_len + 1);
 				if (use_base64)
 					base64_encode(enc_str,
 						      (guchar *)out_str,
@@ -2421,8 +2425,6 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 				else
 					qp_q_encode(enc_str, (guchar *)out_str);
 
-				g_free(out_str);
-
 				/* output MIME-encoded string block */
 				mime_block_len = mimestr_len +
 					strlen(block_encoding) +
@@ -2435,6 +2437,10 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 				srcp += cur_len;
 
 				left -= mime_block_len;
+
+				g_free(enc_str);
+				g_free(out_str);
+				g_free(part_str);
 			}
 
 			LBREAK_IF_REQUIRED(cont, FALSE);
@@ -2560,12 +2566,57 @@ gchar *conv_encode_filename(const gchar *src, const gchar *param_name,
 	return g_string_free(string, FALSE);
 }
 
+static gint conv_copy_file_with_gconvert(const gchar *src, const gchar *dest,
+					 const gchar *encoding)
+{
+	gchar *src_s = NULL;
+	gsize len = 0, dlen = 0;
+	gchar *dest_s;
+	GError *error = NULL;
+
+	g_return_val_if_fail(src != NULL, -1);
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(encoding != NULL, -1);
+
+	if (g_file_get_contents(src, &src_s, &len, &error) == FALSE) {
+		g_warning("conv_copy_utf16_file(): %s: %s", src, error->message);
+		g_error_free(error);
+		return -1;
+	}
+
+	dest_s = g_convert(src_s, len, CS_UTF_8, encoding, NULL, &dlen, &error);
+	if (!dest_s) {
+		g_warning("conv_copy_utf16_file(): %s: %s", src, error->message);
+		g_error_free(error);
+		g_free(src_s);
+		return -1;
+	}
+
+	if (g_file_set_contents(dest, dest_s, dlen, &error) == FALSE) {
+		g_warning("conv_copy_utf16_file(): %s: %s", dest, error->message);
+		g_error_free(error);
+		g_free(dest_s);
+		g_free(src_s);
+		return -1;
+	}
+
+	g_free(dest_s);
+	g_free(src_s);
+	return 0;
+}
+
 gint conv_copy_file(const gchar *src, const gchar *dest, const gchar *encoding)
 {
 	FILE *src_fp, *dest_fp;
 	gchar buf[BUFFSIZE];
 	CodeConverter *conv;
 	gboolean err = FALSE;
+	CharSet charset;
+
+	charset = conv_get_charset_from_str(encoding);
+	if (charset == C_UTF_16 || charset == C_UTF_16BE || charset == C_UTF_16LE) {
+		return conv_copy_file_with_gconvert(src, dest, encoding);
+	}
 
 	if ((src_fp = g_fopen(src, "rb")) == NULL) {
 		FILE_OP_ERROR(src, "fopen");
@@ -2653,6 +2704,7 @@ CharSet conv_check_file_encoding(const gchar *file)
 	CharSet enc;
 	const gchar *enc_str;
 	gboolean is_locale = TRUE, is_utf8 = TRUE;
+	size_t size;
 
 	g_return_val_if_fail(file != NULL, C_AUTO);
 
@@ -2666,6 +2718,75 @@ CharSet conv_check_file_encoding(const gchar *file)
 		return C_AUTO;
 	}
 
+	/* UTF-16 check */
+	if ((size = fread(buf, 2, BUFFSIZE / 2, fp)) > 0) {
+		CharSet guess_enc = C_AUTO;
+
+		debug_print("conv_check_file_encoding: check first %d bytes of file %s\n", size * 2, file);
+
+		/* BOM check */
+		if ((buf[0] & 0xff) == 0xfe && (buf[1] & 0xff) == 0xff) {
+			debug_print("conv_check_file_encoding: UTF-16 BOM (BE) found\n");
+			guess_enc = C_UTF_16; /* UTF-16BE */
+		} else if ((buf[0] & 0xff) == 0xff && (buf[1] & 0xff) == 0xfe) {
+			debug_print("conv_check_file_encoding: UTF-16 BOM (LE) found\n");
+			guess_enc = C_UTF_16; /* UTF-16LE */
+		}
+		if (guess_enc != C_AUTO) {
+			fclose(fp);
+			return guess_enc;
+		}
+
+		/* search UTF-16 CR/LF */
+		if (memchr(buf, 0x00, size * 2) != NULL) {
+			gint i;
+			guchar c1, c2;
+
+			for (i = 0; i < size; i++) {
+				c1 = buf[i * 2] & 0xff;
+				c2 = buf[i * 2 + 1] & 0xff;
+				if (c1 == 0x00 && c2 == 0x0d) { /* UTF-16BE CR */
+					i++;
+					if (i >= size) {
+						break;
+					}
+					c1 = buf[i * 2] & 0xff;
+					c2 = buf[i * 2 + 1] & 0xff;
+					if (c1 == 0x00 && c2 == 0x0a) { /* UTF-16BE LF */
+						guess_enc = C_UTF_16BE;
+						break;
+					}
+				} else if (c1 == 0x0d && c2 == 0x00) { /* UTF-16LE CR */
+					i++;
+					if (i >= size) {
+						break;
+					}
+					c1 = buf[i * 2] & 0xff;
+					c2 = buf[i * 2 + 1] & 0xff;
+					if (c1 == 0x0a && c2 == 0x00) { /* UTF-16LE LF */
+						guess_enc = C_UTF_16LE;
+						break;
+					}
+				} else if (c1 == 0x00 && c2 == 0x0a) { /* UTF-16BE LF */
+					guess_enc = C_UTF_16BE;
+					break;
+				} else if (c1 == 0x0a && c2 == 0x00) { /* UTF-16LE LF */
+					guess_enc = C_UTF_16LE;
+					break;
+				}
+			}
+
+			if (guess_enc != C_AUTO) {
+				debug_print("conv_check_file_encoding: %s detected\n",
+					    conv_get_charset_str(guess_enc));
+				fclose(fp);
+				return guess_enc;
+			}
+		}
+	}
+
+	rewind(fp);
+
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
 		gchar *str;
 		gint error = 0;
