@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2012 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2017 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -341,6 +341,20 @@ struct _PrefsCommon
 	gboolean show_attach_tab;            /* Message - Attachment */
 	gboolean show_attached_files_first;  /* Message - Attachment */
 	gint attach_toolbutton_pos;          /* Message - Attachment */
+
+	gboolean enable_newmsg_notify_window; /* Receive */
+
+	gboolean nofilter_junk_sender_in_book; /* Junk Mail */
+
+	gboolean alt_prefer_html;            /* Message */
+
+	gint save_file_type;
+
+	gint notify_window_period;           /* Receive */
+
+	gint startup_online_mode;            /* Online */
+
+	gint addressbook_col_nickname;
 };
 
 extern PrefsCommon prefs_common;
