@@ -57,6 +57,7 @@ G_DEFINE_TYPE(SylApp, syl_app, G_TYPE_OBJECT);
 enum {
 	INIT_DONE,
 	APP_EXIT,
+	APP_FORCE_EXIT,
 	ADD_MSG,
 	REMOVE_MSG,
 	REMOVE_ALL_MSG,
@@ -98,6 +99,15 @@ static void syl_app_class_init(SylAppClass *klass)
 			     syl_marshal_VOID__VOID,
 			     G_TYPE_NONE,
 			     0);
+	app_signals[APP_FORCE_EXIT] =
+		g_signal_new("app-force-exit",
+			     G_TYPE_FROM_CLASS(gobject_class),
+			     G_SIGNAL_RUN_FIRST,
+			     0,
+			     NULL, NULL,
+			     syl_marshal_VOID__VOID,
+			     G_TYPE_NONE,
+			     0);
 	app_signals[ADD_MSG] =
 		g_signal_new("add-msg",
 			     G_TYPE_FROM_CLASS(gobject_class),
@@ -230,9 +240,6 @@ void syl_init(void)
 #endif
 
 	sock_init();
-#if USE_SSL
-	ssl_init();
-#endif
 
 #ifdef G_OS_UNIX
 	/* ignore SIGPIPE signal for preventing sudden death of program */
@@ -326,9 +333,6 @@ void syl_cleanup(void)
 #endif
 	close_log_file();
 
-#if USE_SSL
-	ssl_done();
-#endif
 	sock_cleanup();
 
 	if (app) {
