@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2013 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2017 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -1730,6 +1730,50 @@ gchar **strsplit_csv(const gchar *str, gchar delim, gint max_tokens)
 	return str_array;
 }
 
+/* Concatenate multiple strings into a CSV (TSV) record.
+ * If delimiter characters or double-quotes appear in the string,
+   it is enclosed by double quotes.
+ * Double quote characters are escaped by another double quote.
+ * Strings must not include line breaks in this implementation.
+ */
+gchar *strconcat_csv(gchar delim, const gchar *field1, ...)
+{
+	va_list args;
+	gchar *s;
+	GString *csv;
+	gint count = 0;
+	const gchar *p;
+
+	g_return_val_if_fail(field1 != NULL, NULL);
+
+	csv = g_string_new("");
+
+	va_start(args, field1);
+	s = (gchar *)field1;
+	while (s) {
+		gboolean add_quote = FALSE;
+
+		if (count > 0)
+			g_string_append_c(csv, delim);
+		if (strchr(s, delim) != NULL || strchr(s, '"') != NULL)
+			add_quote = TRUE;
+		if (add_quote)
+			g_string_append_c(csv, '"');
+		for (p = s; *p != '\0'; p++) {
+			if (*p == '"')
+				g_string_append_c(csv, '"');
+			g_string_append_c(csv, *p);
+		}
+		if (add_quote)
+			g_string_append_c(csv, '"');
+		count++;
+		s = va_arg(args, gchar*);
+	}
+	va_end(args);
+
+	return g_string_free(csv, FALSE);
+}
+
 gchar *get_abbrev_newsgroup_name(const gchar *group, gint len)
 {
 	gchar *abbrev_group;
@@ -2397,7 +2441,7 @@ const gchar *get_domain_name(void)
 
 off_t get_file_size(const gchar *file)
 {
-	struct stat s;
+	GStatBuf s;
 
 	if (g_stat(file, &s) < 0) {
 		FILE_OP_ERROR(file, "stat");
@@ -2509,7 +2553,7 @@ gboolean file_exist(const gchar *file, gboolean allow_fifo)
 		return FALSE;
 
 	if (allow_fifo) {
-		struct stat s;
+		GStatBuf s;
 
 		if (g_stat(file, &s) < 0) {
 			if (ENOENT != errno) FILE_OP_ERROR(file, "stat");
@@ -2717,7 +2761,7 @@ gint remove_expired_files(const gchar *dir, guint hours)
 {
 	GDir *dp;
 	const gchar *dir_name;
-	struct stat s;
+	GStatBuf s;
 	gchar *prev_dir;
 	guint file_no;
 	time_t mtime, now, expire_time;
@@ -2771,7 +2815,7 @@ gint remove_expired_files(const gchar *dir, guint hours)
 
 static gint remove_dir_recursive_real(const gchar *dir)
 {
-	struct stat s;
+	GStatBuf s;
 	GDir *dp;
 	const gchar *dir_name;
 	gchar *prev_dir;
@@ -4209,6 +4253,15 @@ gint execute_open_file(const gchar *file, const gchar *content_type)
 
 	log_print("opening %s - %s\n", file, content_type ? content_type : "");
 
+	argv[1] = file;
+	execute_async(argv);
+#else
+	const gchar *argv[3] = {"xdg-open", NULL, NULL};
+
+	g_return_val_if_fail(file != NULL, -1);
+
+	log_print("opening %s - %s\n", file, content_type ? content_type : "");
+
 	argv[1] = file;
 	execute_async(argv);
 #endif
@@ -4316,7 +4369,7 @@ gint play_sound(const gchar *file, gboolean async)
 	return 0;
 }
 
-time_t remote_tzoffset_sec(const gchar *zone)
+stime_t remote_tzoffset_sec(const gchar *zone)
 {
 	static gchar ustzstr[] = "PSTPDTMSTMDTCSTCDTESTEDT";
 	gchar zone3[4];
@@ -4384,15 +4437,16 @@ time_t remote_tzoffset_sec(const gchar *zone)
 	return remoteoffset;
 }
 
-time_t tzoffset_sec(time_t *now)
+stime_t tzoffset_sec(stime_t *now)
 {
+	time_t now_ = *now;
 	struct tm gmt, *tmp, *lt;
 	gint off;
 
-	tmp = gmtime(now);
+	tmp = gmtime(&now_);
 	g_return_val_if_fail(tmp != NULL, -1);
 	gmt = *tmp;
-	lt = localtime(now);
+	lt = localtime(&now_);
 	g_return_val_if_fail(lt != NULL, -1);
 
 	off = (lt->tm_hour - gmt.tm_hour) * 60 + lt->tm_min - gmt.tm_min;
@@ -4415,16 +4469,17 @@ time_t tzoffset_sec(time_t *now)
 }
 
 /* calculate timezone offset (buf must not be less than 6 bytes) */
-gchar *tzoffset_buf(gchar *buf, time_t *now)
+gchar *tzoffset_buf(gchar *buf, stime_t *now)
 {
+	time_t now_ = *now;
 	struct tm gmt, *tmp, *lt;
 	gint off;
 	gchar sign = '+';
 
-	tmp = gmtime(now);
+	tmp = gmtime(&now_);
 	g_return_val_if_fail(tmp != NULL, NULL);
 	gmt = *tmp;
-	lt = localtime(now);
+	lt = localtime(&now_);
 	g_return_val_if_fail(lt != NULL, NULL);
 
 	off = (lt->tm_hour - gmt.tm_hour) * 60 + lt->tm_min - gmt.tm_min;
@@ -4451,7 +4506,7 @@ gchar *tzoffset_buf(gchar *buf, time_t *now)
 	return buf;
 }
 
-gchar *tzoffset(time_t *now)
+gchar *tzoffset(stime_t *now)
 {
 	static gchar offset_string[6];
 
@@ -4462,17 +4517,18 @@ void get_rfc822_date(gchar *buf, gint len)
 {
 	struct tm *lt;
 	time_t t;
+	stime_t t_;
 	gchar day[4], mon[4];
 	gint dd, hh, mm, ss, yyyy;
 	gchar off[6];
 
-	t = time(NULL);
+	t_ = t = time(NULL);
 	lt = localtime(&t);
 
 	sscanf(asctime(lt), "%3s %3s %d %d:%d:%d %d\n",
 	       day, mon, &dd, &hh, &mm, &ss, &yyyy);
 	g_snprintf(buf, len, "%s, %d %s %d %02d:%02d:%02d %s",
-		   day, dd, mon, yyyy, hh, mm, ss, tzoffset_buf(off, &t));
+		   day, dd, mon, yyyy, hh, mm, ss, tzoffset_buf(off, &t_));
 }
 
 /* just a wrapper to suppress the warning of gcc about %c */
