@@ -24,7 +24,6 @@
 #include <glib.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <dirent.h>
 #include <sys/stat.h>
 #include <math.h>
 #include <setjmp.h>
@@ -34,6 +33,7 @@
 #include "addritem.h"
 #include "addrcache.h"
 #include "addrbook.h"
+#include "utils.h"
 
 #ifndef DEV_STANDALONE
 #include "prefs.h"
@@ -264,7 +264,17 @@ ItemEMail *addrbook_person_remove_email( AddressBookFile *book, ItemPerson *pers
 	g_return_val_if_fail( book != NULL, NULL );
 
 	item = addrcache_person_remove_email( book->addressCache, person, email );
-	if( item ); book->dirtyFlag = TRUE;
+	if( item ) book->dirtyFlag = TRUE;
+	return item;
+}
+
+ItemEMail *addrbook_group_remove_email( AddressBookFile *book, ItemGroup *group, ItemEMail *email ) {
+	ItemEMail *item;
+
+	g_return_val_if_fail( book != NULL, NULL );
+
+	item = addritem_group_remove_email( group, email );
+	if( item ) book->dirtyFlag = TRUE;
 	return item;
 }
 
@@ -1071,7 +1081,7 @@ gint addrbook_write_to( AddressBookFile *book, gchar *newFile ) {
 
 	book->retVal = MGU_OPEN_FILE;
 #ifdef DEV_STANDALONE
-	fp = fopen( fileSpec, "wb" );
+	fp = g_fopen( fileSpec, "wb" );
 	g_free( fileSpec );
 	if( fp ) {
 		fputs( "<?xml version=\"1.0\" ?>\n", fp );
@@ -1276,6 +1286,7 @@ void addrbook_update_address_list( AddressBookFile *book, ItemPerson *person, GL
 				groupEMail = g_list_remove( groupEMail, emailGrp );
 				nodeGrpEM = g_list_next( nodeGrpEM );
 			}
+			group->listEMail = groupEMail;
 
 			/* Move on to next group */
 			nodeGrp = g_list_next( nodeGrp );
@@ -1438,7 +1449,7 @@ void addrbook_update_group_list( AddressBookFile *book, ItemGroup *group, GList
 	oldData = group->listEMail;
 	group->listEMail = listEMail;
 	mgu_clear_list( oldData );
-	oldData = NULL;
+	g_list_free ( oldData );
 	book->dirtyFlag = TRUE;
 }
 
@@ -1626,9 +1637,9 @@ ItemFolder *addrbook_remove_folder_delete( AddressBookFile *book, ItemFolder *fo
 */
 GList *addrbook_get_bookfile_list( AddressBookFile *book ) {
 	gchar *adbookdir;
-	DIR *dp;
-	struct dirent *entry;
-	struct stat statbuf;
+	GDir *dir;
+	const gchar *dir_name;
+	GStatBuf statbuf;
 	gchar buf[ WORK_BUFLEN ];
 	gchar numbuf[ WORK_BUFLEN ];
 	gint len, lenpre, lensuf, lennum;
@@ -1654,7 +1665,7 @@ GList *addrbook_get_bookfile_list( AddressBookFile *book ) {
 	adbookdir = g_strdup( buf );
 	strcat( buf, ADDRBOOK_PREFIX );
 
-	if( ( dp = opendir( adbookdir ) ) == NULL ) {
+	if( ( dir = g_dir_open( adbookdir, 0, NULL ) ) == NULL ) {
 		book->retVal = MGU_OPEN_DIRECTORY;
 		g_free( adbookdir );
 		return NULL;
@@ -1665,18 +1676,18 @@ GList *addrbook_get_bookfile_list( AddressBookFile *book ) {
 	lennum = FILE_NUMDIGITS + lenpre;
 	maxval = -1;
 
-	while( ( entry = readdir( dp ) ) != NULL ) {
+	while( ( dir_name = g_dir_read_name( dir ) ) != NULL ) {
 		gchar *endptr = NULL;
 		gint i;
 		gboolean flg;
 
 		strcpy( buf, adbookdir );
-		strcat( buf, entry->d_name );
-		stat( buf, &statbuf );
+		strcat( buf, dir_name );
+		g_stat( buf, &statbuf );
 		if( S_IFREG & statbuf.st_mode ) {
-			if( strncmp( entry->d_name, ADDRBOOK_PREFIX, lenpre ) == 0 ) {
-				if( strncmp( (entry->d_name) + lennum, ADDRBOOK_SUFFIX, lensuf ) == 0 ) {
-					strncpy( numbuf, (entry->d_name) + lenpre, FILE_NUMDIGITS );
+			if( strncmp( dir_name, ADDRBOOK_PREFIX, lenpre ) == 0 ) {
+				if( strncmp( (dir_name) + lennum, ADDRBOOK_SUFFIX, lensuf ) == 0 ) {
+					strncpy( numbuf, (dir_name) + lenpre, FILE_NUMDIGITS );
 					numbuf[ FILE_NUMDIGITS ] = '\0';
 					flg = TRUE;
 					for( i = 0; i < FILE_NUMDIGITS; i++ ) {
@@ -1690,14 +1701,14 @@ GList *addrbook_get_bookfile_list( AddressBookFile *book ) {
 						val = strtol( numbuf, &endptr, 10 );
 						if( endptr  && val > -1 ) {
 							if( val > maxval ) maxval = val;
-							fileList = g_list_append( fileList, g_strdup( entry->d_name ) );
+							fileList = g_list_append( fileList, g_strdup( dir_name ) );
 						}
 					}
 				}
 			}
 		}
 	}
-	closedir( dp );
+	g_dir_close( dir );
 	g_free( adbookdir );
 
 	book->maxValue = maxval; 
