@@ -242,9 +242,9 @@ static void addressbook_edit_ldap_dialog_create( gboolean *cancelled ) {
 
 	/* Status line */
 	hsbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, BORDER_WIDTH);
+	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, 0);
 	statusbar = gtk_statusbar_new();
-	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, BORDER_WIDTH);
+	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, 0);
 
 	/* Button panel */
 	gtkut_stock_button_set_create(&hbbox, &ok_btn, GTK_STOCK_OK,
@@ -327,7 +327,7 @@ void addressbook_edit_ldap_page_basic( gint pageNum, gchar *pageLbl ) {
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
 	hbox_spin = gtk_hbox_new (FALSE, 8);
-	spinbtn_port_adj = gtk_adjustment_new (389, 1, 65535, 100, 1000, 1000);
+	spinbtn_port_adj = gtk_adjustment_new (389, 1, 65535, 100, 1000, 0);
 	spinbtn_port = gtk_spin_button_new(GTK_ADJUSTMENT (spinbtn_port_adj), 1, 0);
 	gtk_box_pack_start (GTK_BOX (hbox_spin), spinbtn_port, FALSE, FALSE, 0);
 	gtk_widget_set_size_request (spinbtn_port, 64, -1);
@@ -432,7 +432,7 @@ void addressbook_edit_ldap_page_extended( gint pageNum, gchar *pageLbl ) {
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
 	hbox_spin = gtk_hbox_new (FALSE, 8);
-	spinbtn_timeout_adj = gtk_adjustment_new (0, 0, 300, 1, 10, 10);
+	spinbtn_timeout_adj = gtk_adjustment_new (0, 0, 300, 1, 10, 0);
 	spinbtn_timeout = gtk_spin_button_new(GTK_ADJUSTMENT (spinbtn_timeout_adj), 1, 0);
 	gtk_box_pack_start (GTK_BOX (hbox_spin), spinbtn_timeout, FALSE, FALSE, 0);
 	gtk_widget_set_size_request (spinbtn_timeout, 64, -1);
@@ -446,7 +446,7 @@ void addressbook_edit_ldap_page_extended( gint pageNum, gchar *pageLbl ) {
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
 	hbox_spin = gtk_hbox_new (FALSE, 8);
-	spinbtn_maxentry_adj = gtk_adjustment_new (0, 0, 500, 1, 10, 10);
+	spinbtn_maxentry_adj = gtk_adjustment_new (0, 0, 500, 1, 10, 0);
 	spinbtn_maxentry = gtk_spin_button_new(GTK_ADJUSTMENT (spinbtn_maxentry_adj), 1, 0);
 	gtk_box_pack_start (GTK_BOX (hbox_spin), spinbtn_maxentry, FALSE, FALSE, 0);
 	gtk_widget_set_size_request (spinbtn_maxentry, 64, -1);
