@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2012 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2013 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,14 @@
 #include <glib.h>
 #include <gtk/gtkfilechooser.h>
 
+typedef struct _FileselFileType	FileselFileType;
+
+struct _FileselFileType
+{
+	gchar *type;
+	gchar *ext;
+};
+
 gchar *filesel_select_file	(const gchar		*title,
 				 const gchar		*file,
 				 GtkFileChooserAction	 action);
@@ -31,6 +39,10 @@ GSList *filesel_select_files	(const gchar		*title,
 				 GtkFileChooserAction	 action);
 
 gchar *filesel_save_as		(const gchar		*file);
+gchar *filesel_save_as_type	(const gchar		*file,
+				 FileselFileType	*types,
+				 gint			 default_type,
+				 gint			*selected_type);
 gchar *filesel_select_dir	(const gchar		*dir);
 
 #endif /* __FILESEL_H__ */
