@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2011 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2014 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -144,6 +144,48 @@ gboolean gtkut_font_can_load(const gchar *str)
 #endif
 }
 
+static gdouble system_dpi = 0.0;
+
+gdouble gtkut_get_dpi(void)
+{
+	gdouble dp, di;
+
+	if (system_dpi > 0.0)
+		return system_dpi;
+
+	//dpi = gdk_screen_get_resolution(gdk_screen_get_default());
+
+	dp = gdk_screen_get_height(gdk_screen_get_default());
+	di = gdk_screen_get_height_mm(gdk_screen_get_default()) / 25.4;
+	system_dpi = dp / di;
+
+	debug_print("gtkut_get_dpi: dpi: %f\n", system_dpi);
+	return system_dpi;
+}
+
+gdouble gtkut_get_dpi_multiplier(void)
+{
+	gdouble dpi;
+	gdouble mul;
+
+	dpi = gtkut_get_dpi();
+
+	/* 96 / 120 / 144 dpi */
+	if (dpi > 142)
+		mul = 1.5;
+	else if (dpi > 118)
+		mul = 1.25;
+	else
+		mul = 1.0;
+
+	return mul;
+}
+
+void gtkut_set_dpi(gdouble dpi)
+{
+	system_dpi = dpi;
+}
+
 void gtkut_convert_int_to_gdk_color(gint rgbvalue, GdkColor *color)
 {
 	g_return_if_fail(color != NULL);
@@ -171,7 +213,7 @@ void gtkut_stock_button_set_create(GtkWidget **bbox,
 
 	*bbox = gtk_hbutton_box_new();
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(*bbox), GTK_BUTTONBOX_END);
-	gtk_box_set_spacing(GTK_BOX(*bbox), 6);
+	gtk_box_set_spacing(GTK_BOX(*bbox), 6 * gtkut_get_dpi_multiplier());
 
 	if (button3) {
 		*button3 = gtk_button_new_from_stock(label3);
@@ -355,6 +397,11 @@ static void vadjustment_changed(GtkAdjustment *adj, gpointer data)
 
 	gtk_widget_queue_draw(widget);
 }
+#elif defined(__APPLE__)
+static void clist_select_row(GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer data)
+{
+	gtk_widget_queue_draw(GTK_WIDGET(clist));
+}
 #endif
 
 void gtkut_clist_set_redraw(GtkCList *clist)
@@ -364,6 +411,9 @@ void gtkut_clist_set_redraw(GtkCList *clist)
 		g_signal_connect(G_OBJECT(clist->vadjustment), "changed",
 				 G_CALLBACK(vadjustment_changed), clist);
 	}
+#elif defined(__APPLE__)
+	g_signal_connect(G_OBJECT(clist), "select-row",
+			 G_CALLBACK(clist_select_row), NULL);
 #endif
 }
 
@@ -845,7 +895,7 @@ gboolean gtkut_text_buffer_find(GtkTextBuffer *buffer, const GtkTextIter *iter,
 
 	wcs = g_utf8_to_ucs4(str, -1, &items_read, &items_written, &error);
 	if (error != NULL) {
-		g_warning("An error occured while converting a string from UTF-8 to UCS-4: %s\n", error->message);
+		g_warning("An error occurred while converting a string from UTF-8 to UCS-4: %s\n", error->message);
 		g_error_free(error);
 	}
 	if (!wcs || items_written <= 0) return FALSE;
@@ -880,7 +930,7 @@ gboolean gtkut_text_buffer_find_backward(GtkTextBuffer *buffer,
 
 	wcs = g_utf8_to_ucs4(str, -1, &items_read, &items_written, &error);
 	if (error != NULL) {
-		g_warning("An error occured while converting a string from UTF-8 to UCS-4: %s\n", error->message);
+		g_warning("An error occurred while converting a string from UTF-8 to UCS-4: %s\n", error->message);
 		g_error_free(error);
 	}
 	if (!wcs || items_written <= 0) return FALSE;
@@ -961,6 +1011,7 @@ void gtkut_window_popup(GtkWidget *window)
 		gdk_window_move(window->window, new_x, new_y);
 
 	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(window), FALSE);
+	gtk_widget_show(window);
 	gtk_window_present(GTK_WINDOW(window));
 #ifdef G_OS_WIN32
 	/* ensure that the window is displayed at the top */
