@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2017 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -62,7 +62,7 @@
 #define PAGE_ATTRIBUTES            1
 #define PAGE_FINISH                2
 
-#define IMPORTCSV_WIDTH           420
+#define IMPORTCSV_WIDTH           480
 #define IMPORTCSV_HEIGHT          320
 
 #define FIELDS_N_COLS              3
@@ -107,6 +107,7 @@ typedef enum {
 	ATTR_NICK_NAME,
 	ATTR_EMAIL_ADDRESS,
 	ATTR_REMARKS,
+	ATTR_ALIAS,
 
 	N_CSV_ATTRIB
 } ImpCSVAttribIndex;
@@ -122,7 +123,8 @@ static struct _ImpCSVAttrib {
 	{N_("Display Name"),	NULL, 2, TRUE},
 	{N_("Nick Name"),	NULL, 3, TRUE},
 	{N_("E-Mail Address"),	NULL, 4, TRUE},
-	{N_("Remarks"),		NULL, 5, TRUE}
+	{N_("Remarks"),		NULL, 5, TRUE},
+	{N_("Alias"),		NULL, 6, TRUE}
 };
 
 static AddressBookFile *_importedBook_;
@@ -361,6 +363,7 @@ static gint imp_csv_import_data( gchar *csvFile, AddressCache *cache ) {
 	gchar *nickName = NULL;
 	gchar *address = NULL;
 	gchar *remarks = NULL;
+	gchar *alias = NULL;
 	ItemPerson *person;
 	ItemEMail *email;
 	gint count = 0;
@@ -406,6 +409,7 @@ static gint imp_csv_import_data( gchar *csvFile, AddressCache *cache ) {
 		nickName  = imp_csv_attrib[ATTR_NICK_NAME].value;
 		address   = imp_csv_attrib[ATTR_EMAIL_ADDRESS].value;
 		remarks   = imp_csv_attrib[ATTR_REMARKS].value;
+		alias     = imp_csv_attrib[ATTR_ALIAS].value;
 
 		if (!fullName && !firstName && !lastName && address)
 			fullName = address;
@@ -422,6 +426,7 @@ static gint imp_csv_import_data( gchar *csvFile, AddressCache *cache ) {
 			email = addritem_create_item_email();
 			addritem_email_set_address( email, address );
 			addritem_email_set_remarks( email, remarks );
+			addritem_email_set_alias( email, alias );
 			addrcache_id_email( cache, email );
 			addrcache_person_add_email( cache, person, email );
 		}
@@ -878,7 +883,6 @@ static void imp_csv_dialog_create() {
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_widget_set_size_request(window, IMPORTCSV_WIDTH, IMPORTCSV_HEIGHT );
-	gtk_container_set_border_width( GTK_CONTAINER(window), 0 );
 	gtk_window_set_title( GTK_WINDOW(window), _("Import CSV file into Address Book") );
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);	
@@ -911,14 +915,16 @@ static void imp_csv_dialog_create() {
 	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, 0);
 
 	/* Button panel */
-	gtkut_stock_button_set_create(&hbbox, &btnNext, _("Next"),
-				      &btnPrev, _("Prev"),
-				      &btnCancel, GTK_STOCK_CANCEL);
-	gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(hbbox), btnCancel,
-					   TRUE);
-	gtkut_box_set_reverse_order(GTK_BOX(hbbox), FALSE);
-	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(hbbox), 4);
+	if (prefs_common.comply_gnome_hig) {
+		gtkut_stock_button_set_create(&hbbox, &btnNext, _("Next"),
+					      &btnPrev, _("Prev"),
+					      &btnCancel, GTK_STOCK_CANCEL);
+	} else {
+		gtkut_stock_button_set_create(&hbbox, &btnPrev, _("Prev"),
+					      &btnNext, _("Next"),
+					      &btnCancel, GTK_STOCK_CANCEL);
+	}
+	gtk_box_pack_end(GTK_BOX(vnbox), hbbox, FALSE, FALSE, 0);
 	gtk_widget_grab_default(btnNext);
 
 	/* Button handlers */
