@@ -1,5 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
+ * Copyright (C) 1999-2017 Hiroyuki Yamamoto
  * Copyright (C) 2001 Match Grun
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,10 +18,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/*
- * Edit VCard address book data.
- */
-
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -37,22 +34,24 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkbutton.h>
+#include <gtk/gtkhbbox.h>
 #include <gtk/gtkcheckbutton.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkstatusbar.h>
 #include <gtk/gtknotebook.h>
-#include <gtk/gtkfilesel.h>
 #include <gtk/gtkstock.h>
 
 #include "addrbook.h"
 #include "addressbook.h"
 #include "addressitem.h"
+#include "filesel.h"
 #include "gtkutils.h"
 #include "stock_pixmap.h"
 #include "prefs_common.h"
 #include "manage_window.h"
 #include "mgutils.h"
 #include "ldif.h"
+#include "codeconv.h"
 #include "utils.h"
 
 #define IMPORTLDIF_GUESS_NAME "LDIF Import"
@@ -61,7 +60,7 @@
 #define PAGE_ATTRIBUTES            1
 #define PAGE_FINISH                2
 
-#define IMPORTLDIF_WIDTH           380
+#define IMPORTLDIF_WIDTH           480
 #define IMPORTLDIF_HEIGHT          300
 
 #define FIELDS_N_COLS              3
@@ -99,7 +98,6 @@ static struct _ImpLdif_Dlg {
 	gboolean  cancelled;
 } impldif_dlg;
 
-static struct _AddressFileSelection _imp_ldif_file_selector_;
 static AddressBookFile *_importedBook_;
 static AddressIndex *_imp_addressIndex_;
 static LdifFile *_ldifFile_ = NULL;
@@ -329,8 +327,11 @@ static gboolean imp_ldif_file_move() {
 	}
 
 	if( ! errFlag ) {
+		gchar *sFSFile;
 		/* Read attribute list */
-		ldif_set_file( _ldifFile_, sFile );
+		sFSFile = conv_filename_from_utf8( sFile );
+		ldif_set_file( _ldifFile_, sFSFile );
+		g_free( sFSFile );
 		if( ldif_read_tags( _ldifFile_ ) == MGU_SUCCESS ) {
 			/* Load fields */
 			/* ldif_print_file( _ldifFile_, stdout ); */
@@ -420,55 +421,18 @@ static void imp_ldif_cancel( GtkWidget *widget, gpointer data ) {
 	gtk_main_quit();
 }
 
-static void imp_ldif_file_ok( GtkWidget *widget, gpointer data ) {
-	const gchar *sFile;
-	AddressFileSelection *afs;
-	GtkWidget *fileSel;
-
-	afs = ( AddressFileSelection * ) data;
-	fileSel = afs->fileSelector;
-	sFile = gtk_file_selection_get_filename( GTK_FILE_SELECTION(fileSel) );
-
-	afs->cancelled = FALSE;
-	gtk_entry_set_text( GTK_ENTRY(impldif_dlg.file_entry), sFile );
-	gtk_widget_hide( afs->fileSelector );
-	gtk_grab_remove( afs->fileSelector );
-	gtk_widget_grab_focus( impldif_dlg.file_entry );
-}
-
-static void imp_ldif_file_cancel( GtkWidget *widget, gpointer data ) {
-	AddressFileSelection *afs = ( AddressFileSelection * ) data;
-	afs->cancelled = TRUE;
-	gtk_widget_hide( afs->fileSelector );
-	gtk_grab_remove( afs->fileSelector );
-	gtk_widget_grab_focus( impldif_dlg.file_entry );
-}
-
-static void imp_ldif_file_select_create( AddressFileSelection *afs ) {
-	GtkWidget *fileSelector;
-
-	fileSelector = gtk_file_selection_new( _("Select LDIF File") );
-	gtk_file_selection_hide_fileop_buttons( GTK_FILE_SELECTION(fileSelector) );
-	g_signal_connect( G_OBJECT (GTK_FILE_SELECTION(fileSelector)->ok_button),
-			  "clicked", G_CALLBACK (imp_ldif_file_ok), ( gpointer ) afs );
-	g_signal_connect( G_OBJECT (GTK_FILE_SELECTION(fileSelector)->cancel_button),
-			  "clicked", G_CALLBACK (imp_ldif_file_cancel), ( gpointer ) afs );
-	afs->fileSelector = fileSelector;
-	afs->cancelled = TRUE;
-}
-
 static void imp_ldif_file_select( void ) {
-	gchar *sFile;
-	if( ! _imp_ldif_file_selector_.fileSelector )
-		imp_ldif_file_select_create( & _imp_ldif_file_selector_ );
+	gchar *sSelFile;
 
-	sFile = gtk_editable_get_chars( GTK_EDITABLE(impldif_dlg.file_entry), 0, -1 );
-	gtk_file_selection_set_filename(
-		GTK_FILE_SELECTION( _imp_ldif_file_selector_.fileSelector ),
-		sFile );
-	g_free( sFile );
-	gtk_widget_show( _imp_ldif_file_selector_.fileSelector );
-	gtk_grab_add( _imp_ldif_file_selector_.fileSelector );
+	sSelFile = filesel_select_file( _("Select LDIF File"), NULL,
+					GTK_FILE_CHOOSER_ACTION_OPEN );
+	if ( sSelFile ) {
+		gchar *sUTF8File;
+		sUTF8File = conv_filename_to_utf8( sSelFile );
+		gtk_entry_set_text( GTK_ENTRY(impldif_dlg.file_entry), sUTF8File );
+		g_free( sUTF8File );
+		g_free( sSelFile );
+	}
 }
 
 static gint imp_ldif_delete_event( GtkWidget *widget, GdkEventAny *event, gpointer data ) {
@@ -598,6 +562,7 @@ static void imp_ldif_page_fields( gint pageNum, gchar *pageLbl ) {
 			GTK_CLIST(clist_field), FIELD_COL_FIELD, FIELDS_COL_WIDTH_FIELD );
 	gtk_clist_set_column_width(
 			GTK_CLIST(clist_field), FIELD_COL_ATTRIB, FIELDS_COL_WIDTH_ATTRIB );
+	gtkut_clist_set_redraw( GTK_CLIST(clist_field) );
 
 	for( i = 0; i < FIELDS_N_COLS; i++ )
 		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(clist_field)->column[i].button, GTK_CAN_FOCUS);
@@ -733,7 +698,6 @@ static void imp_ldif_dialog_create() {
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_widget_set_size_request(window, IMPORTLDIF_WIDTH, IMPORTLDIF_HEIGHT );
-	gtk_container_set_border_width( GTK_CONTAINER(window), 0 );
 	gtk_window_set_title( GTK_WINDOW(window), _("Import LDIF file into Address Book") );
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);	
@@ -741,6 +705,7 @@ static void imp_ldif_dialog_create() {
 			 G_CALLBACK(imp_ldif_delete_event), NULL);
 	g_signal_connect(G_OBJECT(window), "key_press_event",
 			 G_CALLBACK(imp_ldif_key_pressed), NULL);
+	MANAGE_WINDOW_SIGNALS_CONNECT(window);
 
 	vbox = gtk_vbox_new(FALSE, 4);
 	gtk_widget_show(vbox);
@@ -765,11 +730,16 @@ static void imp_ldif_dialog_create() {
 	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, 0);
 
 	/* Button panel */
-	gtkut_stock_button_set_create(&hbbox, &btnPrev, _("Prev"),
-				      &btnNext, _("Next"),
-				      &btnCancel, GTK_STOCK_CANCEL);
-	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(hbbox), 2);
+	if (prefs_common.comply_gnome_hig) {
+		gtkut_stock_button_set_create(&hbbox, &btnNext, _("Next"),
+					      &btnPrev, _("Prev"),
+					      &btnCancel, GTK_STOCK_CANCEL);
+	} else {
+		gtkut_stock_button_set_create(&hbbox, &btnPrev, _("Prev"),
+					      &btnNext, _("Next"),
+					      &btnCancel, GTK_STOCK_CANCEL);
+	}
+	gtk_box_pack_end(GTK_BOX(vnbox), hbbox, FALSE, FALSE, 0);
 	gtk_widget_grab_default(btnNext);
 
 	/* Button handlers */
@@ -808,7 +778,6 @@ AddressBookFile *addressbook_imp_ldif( AddressIndex *addrIndex ) {
 	if( ! impldif_dlg.window )
 		imp_ldif_create();
 	impldif_dlg.cancelled = FALSE;
-	gtk_widget_show(impldif_dlg.window);
 	manage_window_set_transient(GTK_WINDOW(impldif_dlg.window));
 	gtk_widget_grab_default(impldif_dlg.btnNext);
 
@@ -834,6 +803,9 @@ AddressBookFile *addressbook_imp_ldif( AddressIndex *addrIndex ) {
 	impldif_dlg.fileName = NULL;
 
 	_ldifFile_ = ldif_create();
+
+	gtk_widget_show(impldif_dlg.window);
+
 	gtk_main();
 	gtk_widget_hide(impldif_dlg.window);
 	ldif_free( _ldifFile_ );
@@ -849,6 +821,62 @@ AddressBookFile *addressbook_imp_ldif( AddressIndex *addrIndex ) {
 	return _importedBook_;
 }
 
+AddressBookFile *addressbook_imp_ldif_file( AddressIndex *addrIndex,
+					    const gchar *file,
+					    const gchar *book_name ) {
+	gchar *fsfile;
+	GList *node, *list;
+	gboolean ret = FALSE;
+
+	g_return_val_if_fail(addrIndex != NULL, NULL);
+	g_return_val_if_fail(file != NULL, NULL);
+	g_return_val_if_fail(book_name != NULL, NULL);
+
+	debug_print("addressbook_imp_ldif_file: file: %s name: %s\n",
+		    file, book_name);
+
+	_importedBook_ = NULL;
+	_imp_addressIndex_ = addrIndex;
+	_ldifFile_ = ldif_create();
+
+	fsfile = conv_filename_from_utf8( file );
+	ldif_set_file(_ldifFile_, fsfile);
+	g_free( fsfile );
+
+	if( ldif_read_tags( _ldifFile_ ) != MGU_SUCCESS )
+		goto finish;
+	list = ldif_get_fieldlist( _ldifFile_ );
+	node = list;
+	while( node ) {
+		Ldif_FieldRec *rec = node->data;
+		if( ! rec->reserved ) {
+			if( g_ascii_strcasecmp( rec->tagName, "dn" ) != 0 ) {
+				rec->selected = TRUE;
+			}
+		}
+		node = g_list_next( node );
+	}
+	g_list_free( list );
+
+	g_free( impldif_dlg.nameBook );
+	impldif_dlg.nameBook = g_strdup(book_name);
+
+	ret = imp_ldif_field_move();
+
+	g_free( impldif_dlg.nameBook );
+	impldif_dlg.nameBook = NULL;
+
+finish:
+	ldif_free( _ldifFile_ );
+	_ldifFile_ = NULL;
+	_imp_addressIndex_ = NULL;
+
+	if (ret)
+		debug_print("addressbook_imp_ldif_file: import succeeded\n");
+
+	return _importedBook_;
+}
+
 /*
 * End of Source.
 */
