@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,11 +30,13 @@
 
 #include "folder.h"
 #include "prefs.h"
+#include "prefs_ui.h"
 #include "prefs_folder_item.h"
 #include "prefs_account.h"
 #include "account.h"
 #include "manage_window.h"
 #include "folderview.h"
+#include "mainwindow.h"
 #include "inc.h"
 #include "menu.h"
 
@@ -47,7 +49,8 @@ struct _PrefsFolderItemDialog
 
 	/* General */
 	GtkWidget *name_entry;
-	GtkWidget *path_entry;
+	GtkWidget *id_label;
+	GtkWidget *path_label;
 	GtkWidget *type_optmenu;
 
 	GtkWidget *trim_summary_subj_chkbtn;
@@ -146,7 +149,8 @@ static void prefs_folder_item_general_create(PrefsFolderItemDialog *dialog)
 	GtkWidget *hbox;
 	GtkWidget *label;
 	GtkWidget *name_entry;
-	GtkWidget *path_entry;
+	GtkWidget *id_label;
+	GtkWidget *path_label;
 	GtkWidget *optmenu;
 	GtkWidget *optmenu_menu;
 	GtkWidget *menuitem;
@@ -155,14 +159,11 @@ static void prefs_folder_item_general_create(PrefsFolderItemDialog *dialog)
 	GtkWidget *trim_compose_subj_chkbtn;
 	GtkStyle *style;
 
-	style = gtk_style_copy(gtk_widget_get_style(dialog->dialog->window));
-	style->base[GTK_STATE_NORMAL] = style->bg[GTK_STATE_NORMAL];
-
 	vbox = gtk_vbox_new(FALSE, VSPACING);
 	gtk_container_add(GTK_CONTAINER(dialog->dialog->notebook), vbox);
 	gtk_container_set_border_width(GTK_CONTAINER (vbox), VBOX_BORDER);
 
-	table = gtk_table_new(3, 2, FALSE);
+	table = gtk_table_new(4, 2, FALSE);
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 8);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
@@ -175,31 +176,53 @@ static void prefs_folder_item_general_create(PrefsFolderItemDialog *dialog)
 	name_entry = gtk_entry_new();
 	gtk_editable_set_editable(GTK_EDITABLE(name_entry), FALSE);
 	gtk_widget_set_size_request(name_entry, 200, -1);
-	gtk_widget_set_style(name_entry, style);
+	style = gtk_widget_get_style(dialog->dialog->window);
+	gtk_widget_modify_base(name_entry, GTK_STATE_NORMAL,
+			       &style->bg[GTK_STATE_NORMAL]);
 	gtk_table_attach(GTK_TABLE(table), name_entry, 1, 2, 0, 1,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
-	label = gtk_label_new(_("Path"));
+	label = gtk_label_new(_("Identifier"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
 			 GTK_FILL, 0, 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
-	path_entry = gtk_entry_new();
-	gtk_editable_set_editable(GTK_EDITABLE(path_entry), FALSE);
-	gtk_widget_set_size_request(path_entry, 200, -1);
-	gtk_widget_set_style(path_entry, style);
-	gtk_table_attach(GTK_TABLE(table), path_entry, 1, 2, 1, 2,
+	id_label = gtk_label_new("");
+	gtk_label_set_selectable(GTK_LABEL(id_label), TRUE);
+	gtk_misc_set_alignment(GTK_MISC(id_label), 0, 0.5);
+	gtk_label_set_justify(GTK_LABEL(id_label), GTK_JUSTIFY_LEFT);
+#if GTK_CHECK_VERSION(2, 6, 0)
+	gtk_label_set_ellipsize(GTK_LABEL(id_label), PANGO_ELLIPSIZE_MIDDLE);
+#endif
+	gtk_table_attach(GTK_TABLE(table), id_label, 1, 2, 1, 2,
+			 GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+			 GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
+
+	label = gtk_label_new(_("Path"));
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
+			 GTK_FILL, 0, 0, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+
+	path_label = gtk_label_new("");
+	gtk_label_set_selectable(GTK_LABEL(path_label), TRUE);
+	gtk_misc_set_alignment(GTK_MISC(path_label), 0, 0.5);
+	gtk_label_set_justify(GTK_LABEL(path_label), GTK_JUSTIFY_LEFT);
+#if GTK_CHECK_VERSION(2, 6, 0)
+	gtk_label_set_ellipsize(GTK_LABEL(path_label), PANGO_ELLIPSIZE_MIDDLE);
+#endif
+	gtk_table_attach(GTK_TABLE(table), path_label, 1, 2, 2, 3,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
 	label = gtk_label_new(_("Type"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
 			 GTK_FILL, 0, 0, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
 	hbox = gtk_hbox_new(FALSE, 8);
 	gtk_widget_show(hbox);
-	gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 2, 3,
+	gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 3, 4,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
@@ -214,6 +237,7 @@ static void prefs_folder_item_general_create(PrefsFolderItemDialog *dialog)
 	MENUITEM_ADD(optmenu_menu, menuitem, _("Drafts"), F_DRAFT);
 	MENUITEM_ADD(optmenu_menu, menuitem, _("Queue") , F_QUEUE);
 	MENUITEM_ADD(optmenu_menu, menuitem, _("Trash") , F_TRASH);
+	MENUITEM_ADD(optmenu_menu, menuitem, _("Junk")  , F_JUNK);
 
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), optmenu_menu);
 
@@ -225,8 +249,16 @@ static void prefs_folder_item_general_create(PrefsFolderItemDialog *dialog)
 	PACK_CHECK_BUTTON(vbox2, trim_compose_subj_chkbtn,
 			  _("Delete [...] or (...) at the beginning of subject on reply"));
 
+	if (!dialog->item->parent) {
+		gtk_widget_set_sensitive(optmenu, FALSE);
+		gtk_widget_set_sensitive(vbox2, FALSE);
+	}
+	if (dialog->item->stype == F_VIRTUAL)
+		gtk_widget_set_sensitive(optmenu, FALSE);
+
 	dialog->name_entry = name_entry;
-	dialog->path_entry = path_entry;
+	dialog->id_label = id_label;
+	dialog->path_label = path_label;
 	dialog->type_optmenu = optmenu;
 	dialog->trim_summary_subj_chkbtn = trim_summary_subj_chkbtn;
 	dialog->trim_compose_subj_chkbtn = trim_compose_subj_chkbtn;
@@ -355,6 +387,11 @@ static void prefs_folder_item_compose_create(PrefsFolderItemDialog *dialog)
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
+	if (!dialog->item->parent) {
+		gtk_widget_set_sensitive(frame, FALSE);
+		gtk_widget_set_sensitive(ac_apply_sub_chkbtn, FALSE);
+	}
+
 	dialog->account_optmenu     = optmenu;
 	dialog->ac_apply_sub_chkbtn = ac_apply_sub_chkbtn;
 	dialog->to_entry            = to_entry;
@@ -374,6 +411,8 @@ static void prefs_folder_item_set_dialog(PrefsFolderItemDialog *dialog)
 	GtkWidget *menuitem;
 	GtkOptionMenu *optmenu;
 	gchar *id;
+	gchar *path;
+	gchar *utf8_path;
 	GList *cur;
 	SpecialFolderItemType type;
 	gint n;
@@ -384,9 +423,15 @@ static void prefs_folder_item_set_dialog(PrefsFolderItemDialog *dialog)
 	SET_ENTRY(name_entry, name);
 
 	id = folder_item_get_identifier(dialog->item);
-	gtk_entry_set_text(GTK_ENTRY(dialog->path_entry), id);
+	gtk_label_set_text(GTK_LABEL(dialog->id_label), id);
 	g_free(id);
 
+	path = folder_item_get_path(dialog->item);
+	utf8_path = conv_filename_to_utf8(path);
+	gtk_label_set_text(GTK_LABEL(dialog->path_label), utf8_path);
+	g_free(utf8_path);
+	g_free(path);
+
 	optmenu = GTK_OPTION_MENU(dialog->type_optmenu);
 	menu = gtk_option_menu_get_menu(optmenu);
 	for (cur = GTK_MENU_SHELL(menu)->children, n = 0;
@@ -447,6 +492,7 @@ void prefs_folder_item_destroy(PrefsFolderItemDialog *dialog)
 	g_free(dialog->dialog);
 	g_free(dialog);
 
+	main_window_popup(main_window_get());
 	inc_unlock();
 }
 
@@ -483,7 +529,7 @@ static void prefs_folder_item_apply_cb(GtkWidget *widget,
 	type = (SpecialFolderItemType)
 		g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID);
 
-	if (item->stype != type) {
+	if (item->stype != type && item->stype != F_VIRTUAL) {
 		switch (type) {
 		case F_NORMAL:
 			break;
@@ -517,6 +563,15 @@ static void prefs_folder_item_apply_cb(GtkWidget *widget,
 			prev_item = folder->trash;
 			folder->trash = item;
 			break;
+		case F_JUNK:
+			prev_item = folder_get_junk(folder);
+			if (prev_item)
+				prev_item->stype = F_NORMAL;
+			folder_set_junk(folder, item);
+			break;
+		default:
+			type = item->stype;
+			break;
 		}
 
 		item->stype = type;
@@ -542,8 +597,13 @@ static void prefs_folder_item_apply_cb(GtkWidget *widget,
 	else
 		item->account = NULL;
 
-	item->ac_apply_sub = gtk_toggle_button_get_active
-		(GTK_TOGGLE_BUTTON(dialog->ac_apply_sub_chkbtn));
+	if (!item->parent && item->account)
+		item->ac_apply_sub = TRUE;
+	else if (item->account)
+		item->ac_apply_sub = gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON(dialog->ac_apply_sub_chkbtn));
+	else
+		item->ac_apply_sub = FALSE;
 
 	SET_DATA_FROM_ENTRY(to_entry, auto_to);
 	item->use_auto_to_on_reply = gtk_toggle_button_get_active
