@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2008 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,6 +53,7 @@
 #include "alertpanel.h"
 #include "manage_window.h"
 #include "socket.h"
+#include "socks.h"
 #include "utils.h"
 #include "inc.h"
 #include "mainwindow.h"
@@ -69,6 +70,7 @@ struct _SendProgressDialog
 {
 	ProgressDialog *dialog;
 	Session *session;
+	gboolean show_dialog;
 	gboolean cancelled;
 };
 
@@ -212,7 +214,7 @@ QueueInfo *send_get_queue_info(const gchar *file)
 	return qinfo;
 }
 
-gint send_get_queue_contents(QueueInfo *qinfo, const gchar *dest)
+static gint send_get_queue_contents(QueueInfo *qinfo, const gchar *dest)
 {
 	FILE *fp;
 	glong pos;
@@ -243,6 +245,42 @@ gint send_get_queue_contents(QueueInfo *qinfo, const gchar *dest)
 	return 0;
 }
 
+static gint send_save_queued_message(QueueInfo *qinfo, gboolean filter_msgs)
+{
+	FolderItem *outbox;
+	gboolean drop_done = FALSE;
+	gchar tmp[MAXPATHLEN + 1];
+
+	g_snprintf(tmp, sizeof(tmp), "%s%ctmpmsg.out.%08x",
+		   get_rc_dir(), G_DIR_SEPARATOR, g_random_int());
+
+	if (send_get_queue_contents(qinfo, tmp) < 0)
+		return -1;
+
+	if (filter_msgs) {
+		FilterInfo *fltinfo;
+
+		fltinfo = filter_info_new();
+		fltinfo->account = qinfo->ac;
+		fltinfo->flags.perm_flags = 0;
+		fltinfo->flags.tmp_flags = MSG_RECEIVED;
+
+		filter_apply(prefs_common.fltlist, tmp, fltinfo);
+
+		drop_done = fltinfo->drop_done;
+		filter_info_free(fltinfo);
+	}
+
+	if (!drop_done) {
+		outbox = account_get_special_folder(qinfo->ac, F_OUTBOX);
+		procmsg_save_to_outbox(outbox, tmp);
+	}
+
+	g_unlink(tmp);
+
+	return 0;
+}
+
 void send_queue_info_free(QueueInfo *qinfo)
 {
 	if (qinfo == NULL) return;
@@ -340,7 +378,6 @@ gint send_message_queue_all(FolderItem *queue, gboolean save_msgs,
 		gchar *file;
 		MsgInfo *msginfo = (MsgInfo *)cur->data;
 		QueueInfo *qinfo;
-		gchar tmp[MAXPATHLEN + 1];
 
 		file = procmsg_get_message_file(msginfo);
 		if (!file)
@@ -361,37 +398,8 @@ gint send_message_queue_all(FolderItem *queue, gboolean save_msgs,
 		else if (qinfo->forward_targets)
 			send_message_set_forward_flags(qinfo->forward_targets);
 
-		g_snprintf(tmp, sizeof(tmp), "%s%ctmpmsg.out.%08x",
-			   get_rc_dir(), G_DIR_SEPARATOR, g_random_int());
-
-		if (send_get_queue_contents(qinfo, tmp) == 0) {
-			if (save_msgs) {
-				FolderItem *outbox;
-				gboolean drop_done = FALSE;
-
-				if (filter_msgs) {
-					FilterInfo *fltinfo;
-
-					fltinfo = filter_info_new();
-					fltinfo->account = qinfo->ac;
-					fltinfo->flags.perm_flags = 0;
-					fltinfo->flags.tmp_flags = MSG_RECEIVED;
-
-					filter_apply(prefs_common.fltlist, tmp,
-						     fltinfo);
-
-					drop_done = fltinfo->drop_done;
-					filter_info_free(fltinfo);
-				}
-
-				if (!drop_done) {
-					outbox = account_get_special_folder
-						(qinfo->ac, F_OUTBOX);
-					procmsg_save_to_outbox(outbox, tmp);
-				}
-			}
-			g_unlink(tmp);
-		}
+		if (save_msgs)
+			send_save_queued_message(qinfo, filter_msgs);
 
 		send_queue_info_free(qinfo);
 		g_free(file);
@@ -614,6 +622,7 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 {
 	Session *session;
 	SMTPSession *smtp_session;
+	SocksInfo *socks_info = NULL;
 	FILE *out_fp;
 	gushort port;
 	SendProgressDialog *dialog;
@@ -731,12 +740,25 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 
 	session_set_timeout(session, prefs_common.io_timeout_secs * 1000);
 
+	if (ac_prefs->use_socks && ac_prefs->use_socks_for_send) {
+		socks_info = socks_info_new(ac_prefs->socks_type,
+					    ac_prefs->proxy_host,
+					    ac_prefs->proxy_port,
+					    ac_prefs->use_proxy_auth
+						? ac_prefs->proxy_name : NULL,
+					    ac_prefs->use_proxy_auth
+						? ac_prefs->proxy_pass : NULL);
+	}
+
 	inc_lock();
 
-	if (session_connect(session, ac_prefs->smtp_server, port) < 0) {
-		manage_window_focus_in(dialog->dialog->window, NULL, NULL);
+	if (session_connect_full(session, ac_prefs->smtp_server, port,
+				 socks_info) < 0) {
+		if (dialog->show_dialog)
+			manage_window_focus_in(dialog->dialog->window, NULL, NULL);
 		send_put_error(session);
-		manage_window_focus_out(dialog->dialog->window, NULL, NULL);
+		if (dialog->show_dialog)
+			manage_window_focus_out(dialog->dialog->window, NULL, NULL);
 		session_destroy(session);
 		send_progress_dialog_destroy(dialog);
 		inc_unlock();
@@ -747,11 +769,15 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 
 	while (session_is_connected(session) && dialog->cancelled == FALSE)
 		gtk_main_iteration();
+	log_window_flush();
 
 	if (SMTP_SESSION(session)->error_val == SM_AUTHFAIL) {
 		if (ac_prefs->smtp_userid && ac_prefs->tmp_smtp_pass) {
 			g_free(ac_prefs->tmp_smtp_pass);
 			ac_prefs->tmp_smtp_pass = NULL;
+		} else if (!ac_prefs->smtp_userid && ac_prefs->tmp_pass) {
+			g_free(ac_prefs->tmp_pass);
+			ac_prefs->tmp_pass = NULL;
 		}
 		ret = -1;
 	} else if (session->state == SESSION_EOF &&
@@ -774,9 +800,11 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 		ret = -1;
 
 	if (ret == -1) {
-		manage_window_focus_in(dialog->dialog->window, NULL, NULL);
+		if (dialog->show_dialog)
+			manage_window_focus_in(dialog->dialog->window, NULL, NULL);
 		send_put_error(session);
-		manage_window_focus_out(dialog->dialog->window, NULL, NULL);
+		if (dialog->show_dialog)
+			manage_window_focus_out(dialog->dialog->window, NULL, NULL);
 	}
 
 	session_destroy(session);
@@ -795,19 +823,24 @@ static gint send_recv_message(Session *session, const gchar *msg, gpointer data)
 
 	g_return_val_if_fail(dialog != NULL, -1);
 
+	gdk_threads_enter();
+
 	switch (smtp_session->state) {
 	case SMTP_READY:
 	case SMTP_CONNECTED:
+		gdk_threads_leave();
 		return 0;
 	case SMTP_HELO:
 		g_snprintf(buf, sizeof(buf), _("Sending HELO..."));
 		state_str = _("Authenticating");
-		statusbar_print_all(_("Sending message..."));
+		statusbar_print_all(_("Sending message via %s:%d..."),
+				    session->server, session->port);
 		break;
 	case SMTP_EHLO:
 		g_snprintf(buf, sizeof(buf), _("Sending EHLO..."));
 		state_str = _("Authenticating");
-		statusbar_print_all(_("Sending message..."));
+		statusbar_print_all(_("Sending message via %s:%d..."),
+				    session->server, session->port);
 		break;
 	case SMTP_AUTH:
 		g_snprintf(buf, sizeof(buf), _("Authenticating..."));
@@ -832,14 +865,18 @@ static gint send_recv_message(Session *session, const gchar *msg, gpointer data)
 		break;
 	case SMTP_ERROR:
 		g_warning("send: error: %s\n", msg);
+		gdk_threads_leave();
 		return 0;
 	default:
+		gdk_threads_leave();
 		return 0;
 	}
 
 	progress_dialog_set_label(dialog->dialog, buf);
 	progress_dialog_set_row_status(dialog->dialog, 0, state_str);
 
+	gdk_threads_leave();
+
 	return 0;
 }
 
@@ -855,6 +892,8 @@ static gint send_send_data_progressive(Session *session, guint cur_len,
 	    SMTP_SESSION(session)->state != SMTP_EOM)
 		return 0;
 
+	gdk_threads_enter();
+
 	g_snprintf(buf, sizeof(buf), _("Sending message (%d / %d bytes)"),
 		   cur_len, total_len);
 	progress_dialog_set_label(dialog->dialog, buf);
@@ -866,6 +905,7 @@ static gint send_send_data_progressive(Session *session, guint cur_len,
 #ifdef G_OS_WIN32
 	GTK_EVENTS_FLUSH();
 #endif
+	gdk_threads_leave();
 
 	return 0;
 }
@@ -877,6 +917,7 @@ static gint send_send_data_finished(Session *session, guint len, gpointer data)
 	g_return_val_if_fail(dialog != NULL, -1);
 
 	send_send_data_progressive(session, len, len, dialog);
+
 	return 0;
 }
 
@@ -894,12 +935,15 @@ static SendProgressDialog *send_progress_dialog_create(void)
 			 G_CALLBACK(send_cancel_button_cb), dialog);
 	g_signal_connect(G_OBJECT(progress->window), "delete_event",
 			 G_CALLBACK(gtk_true), NULL);
-	gtk_window_set_modal(GTK_WINDOW(progress->window), TRUE);
+	/* gtk_window_set_modal(GTK_WINDOW(progress->window), TRUE); */
 	manage_window_set_transient(GTK_WINDOW(progress->window));
 
 	progress_dialog_set_value(progress, 0.0);
 
-	gtk_widget_show_now(progress->window);
+	if (prefs_common.show_send_dialog) {
+		dialog->show_dialog = TRUE;
+		gtk_widget_show_now(progress->window);
+	}
 
 	dialog->dialog = progress;
 
@@ -953,7 +997,11 @@ static void send_put_error(Session *session)
 	default:
 		switch (session->state) {
 		case SESSION_ERROR:
-			if (SMTP_SESSION(session)->state == SMTP_READY) {
+			if (session_get_error(session) == SESSION_ERROR_LOOKUP) {
+				log_msg = _("Server not found.");
+				err_msg = g_strdup_printf
+					(_("SMTP server not found: %s:%d"), session->server, session->port);
+			} else if (SMTP_SESSION(session)->state == SMTP_READY) {
 				log_msg = _("Can't connect to SMTP server.");
 				err_msg = g_strdup_printf
 					(_("Can't connect to SMTP server: %s:%d"), session->server, session->port);
