@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,6 +42,8 @@ GtkWidget *statusbar_create(void)
 	gtk_widget_set_size_request(statusbar, 1, -1);
 	statusbar_list = g_list_append(statusbar_list, statusbar);
 
+	set_log_show_status_func(statusbar_puts_all);
+
 	return statusbar;
 }
 
@@ -49,12 +51,9 @@ void statusbar_puts(GtkStatusbar *statusbar, const gchar *str)
 {
 	gint cid;
 	gchar *buf;
-	gchar *tmp;
 
-	tmp = g_strdup(str);
-	strretchomp(tmp);
-	buf = trim_string(tmp, 76);
-	g_free(tmp);
+	buf = g_strdup(str);
+	strretchomp(buf);
 
 	cid = gtk_statusbar_get_context_id(statusbar, "Standard Output");
 	gtk_statusbar_pop(statusbar, cid);
@@ -68,8 +67,10 @@ void statusbar_puts_all(const gchar *str)
 {
 	GList *cur;
 
+	gdk_threads_enter();
 	for (cur = statusbar_list; cur != NULL; cur = cur->next)
 		statusbar_puts(GTK_STATUSBAR(cur->data), str);
+	gdk_threads_leave();
 }
 
 void statusbar_print(GtkStatusbar *statusbar, const gchar *format, ...)
