@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2012 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2013 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -500,6 +500,8 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 		return;
 	}
 
+	debug_print("textview_show_message: displaying: %s\n", file);
+
 	charset = textview_get_src_encoding(textview, mimeinfo);
 	textview_set_font(textview, charset);
 	textview_clear(textview);
@@ -512,8 +514,6 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 
 		gtk_text_buffer_get_end_iter(buffer, &iter);
 		textview->body_pos = gtk_text_iter_get_offset(&iter);
-		debug_print("textview_show_message: body_pos: %d\n",
-			    textview->body_pos);
 	} else {
 		gtk_text_buffer_get_end_iter(buffer, &iter);
 		mark = gtk_text_buffer_get_mark(buffer, "attach-file-pos");
@@ -632,8 +632,6 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 
 		gtk_text_buffer_get_end_iter(buffer, &iter);
 		textview->body_pos = gtk_text_iter_get_offset(&iter);
-		debug_print("textview_show_part: body_pos: %d\n",
-			    textview->body_pos);
 		if (!mimeinfo->main)
 			gtk_text_buffer_insert(buffer, &iter, "\n", 1);
 	} else {
@@ -961,6 +959,7 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		g_snprintf(buf, sizeof(buf), "%s (%s)",
 			   mimeinfo->content_type,
 			   to_human_readable(mimeinfo->content_size));
+		debug_print("textview_add_part: adding: %s\n", buf);
 		gtk_text_buffer_insert(buffer, &iter, "\n", 1);
 		textview_add_part_widget(textview, &iter, mimeinfo, buf);
 		gtk_text_buffer_get_end_iter(buffer, &iter);
@@ -994,6 +993,8 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 			   mimeinfo->content_type,
 			   to_human_readable(mimeinfo->content_size));
 
+	debug_print("textview_add_part: adding: %s\n", buf);
+
 	if (mimeinfo->mime_type != MIME_TEXT &&
 	    mimeinfo->mime_type != MIME_TEXT_HTML) {
 		if (mimeinfo->mime_type == MIME_IMAGE &&
@@ -1025,6 +1026,13 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 				return;
 			}
 
+			{
+				GdkPixbuf *rotated;
+
+				rotated = imageview_get_rotated_pixbuf(pixbuf);
+				g_object_unref(pixbuf);
+				pixbuf = rotated;
+			}
 			if (prefs_common.resize_image) {
 				GdkPixbuf *scaled;
 
@@ -1076,9 +1084,9 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	} else {
 		/* text part */
 		gtk_text_buffer_insert(buffer, &iter, "\n", 1);
-		if (!mimeinfo->main &&
-		    mimeinfo->parent &&
-		    mimeinfo->parent->children != mimeinfo)
+		if (mimeinfo->mime_type == MIME_TEXT_HTML ||
+		    (!mimeinfo->main && mimeinfo->parent &&
+		     mimeinfo->parent->children != mimeinfo))
 			textview_add_part_widget(textview, &iter, mimeinfo, buf);
 		textview_write_body(textview, mimeinfo, fp, charset);
 	}
@@ -1142,13 +1150,34 @@ static void textview_add_parts(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	level = mimeinfo->level;
 
 	for (;;) {
-		textview_add_part(textview, mimeinfo, fp);
-		if (mimeinfo->parent && mimeinfo->parent->content_type &&
-		    !g_ascii_strcasecmp(mimeinfo->parent->content_type,
-					"multipart/alternative"))
-			mimeinfo = mimeinfo->parent->next;
-		else
-			mimeinfo = procmime_mimeinfo_next(mimeinfo);
+		if (mimeinfo->mime_type == MIME_MULTIPART &&
+		    mimeinfo->content_type &&
+		    !g_ascii_strcasecmp(mimeinfo->content_type,
+					"multipart/alternative")) {
+			MimeInfo *preferred_part = mimeinfo->children;
+			MimeInfo *child;
+
+			if (prefs_common.alt_prefer_html) {
+				for (child = mimeinfo->children; child != NULL; child = child->next) {
+					if (child->mime_type == MIME_TEXT_HTML) {
+						preferred_part = child;
+						break;
+					}
+				}
+			}
+
+			if (preferred_part) {
+				textview_add_part(textview, preferred_part, fp);
+				mimeinfo = preferred_part;
+				while (mimeinfo->next)
+					mimeinfo = mimeinfo->next;
+			}
+		} else {
+			textview_add_part(textview, mimeinfo, fp);
+		}
+
+		mimeinfo = procmime_mimeinfo_next(mimeinfo);
+
 		if (!mimeinfo || mimeinfo->level <= level)
 			break;
 	}
@@ -1594,6 +1623,9 @@ static void textview_write_link(TextView *textview, const gchar *str,
 	if (bufp > buf)
 		gtk_text_buffer_insert(buffer, &iter, buf, bufp - buf);
 
+	if (gtk_text_iter_ends_tag(&iter, textview->link_tag))
+		gtk_text_buffer_insert(buffer, &iter, " ", 1);
+
 	r_uri = g_new(RemoteURI, 1);
 	r_uri->uri = g_strstrip(g_strdup(uri));
 	r_uri->filename = NULL;
@@ -2726,7 +2758,6 @@ static void textview_uri_list_update_offsets(TextView *textview, gint start, gin
 		RemoteURI *uri = (RemoteURI *)cur->data;
 
 		if (uri->start >= start) {
-			debug_print("textview_uri_list_update_offsets: update: (%d, %d) -> (%d, %d)\n", uri->start, uri->end, uri->start + add, uri->end + add);
 			uri->start += add;
 			uri->end += add;
 		}
